using System;
using System.IO;
using System.Text;
using System.Collections;
using System.Collections.Specialized;


namespace CsSoundStudy
{
	public class WaveFileReader : IDisposable
	{
		BinaryReader reader;
		riffChunk mainfile;
		fmtChunk format;
		factChunk fact;
		dataChunk data;

#region General Utilities

		public WaveFileReader(string filename)
		{
			reader = new BinaryReader(new FileStream(filename, FileMode.Open, FileAccess.Read, FileShare.Read));
      reader.BaseStream.Seek(0, SeekOrigin.Begin);
		}

		public long GetPosition()
		{
			return reader.BaseStream.Position;
		}

		public string GetChunkName()
		{
			return new string(reader.ReadChars(4));
		}

		public void AdvanceToNext()
		{
			long NextOffset = (long) reader.ReadUInt32(); 
      reader.BaseStream.Seek(NextOffset,SeekOrigin.Current);
		}
#endregion
#region Header Extraction Methods

		public riffChunk ReadMainFileHeader()
		{
			mainfile = new riffChunk();

			mainfile.sGroupID = new string(reader.ReadChars(4));
			mainfile.dwFileLength = reader.ReadUInt32()+8;
			mainfile.sRiffType = new string(reader.ReadChars(4));
			return mainfile;
		}

		public fmtChunk ReadFormatHeader()
		{
			format = new fmtChunk();
			format.sChunkID = "fmt ";
			format.dwChunkSize = reader.ReadUInt32();
			format.wFormatTag = reader.ReadUInt16();
			format.wChannels = reader.ReadUInt16();
			format.dwSamplesPerSec = reader.ReadUInt32();
			format.dwAvgBytesPerSec = reader.ReadUInt32();
			format.wBlockAlign = reader.ReadUInt16();
			format.dwBitsPerSample = reader.ReadUInt32();
     	return format;
		} 

		public factChunk ReadFactHeader()
		{
			fact = new factChunk();

			fact.sChunkID = "fact";
			fact.dwChunkSize = reader.ReadUInt32();
			fact.dwNumSamples = reader.ReadUInt32();
			return fact;
		} 
		public dataChunk ReadDataHeader()
		{
			data = new dataChunk();

			data.sChunkID = "data";
			data.dwChunkSize = reader.ReadUInt32();
			data.lFilePosition = reader.BaseStream.Position;
			if (!fact.Equals(null))
				data.dwNumSamples = fact.dwNumSamples;
			else
				data.dwNumSamples = data.dwChunkSize / (format.dwBitsPerSample/8 * format.wChannels);
			data.dwMinLength = (data.dwChunkSize / format.dwAvgBytesPerSec) / 60;
			data.dSecLength = ((double)data.dwChunkSize / (double)format.dwAvgBytesPerSec) - (double)data.dwMinLength*60;
			return data;
		} 

#endregion

#region IDisposable Members

		public void Dispose() 
		{
			if(reader != null) 
				reader.Close();
		}

#endregion


	}
}
